local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local _

-- Implement a dummy TTD function if we do not get one from another module
if type(s.TimeToDie) ~= 'function'  then
	a.print(L["No TimeToDie function found, using a dummy function. Consider installing Pusikas' Spellflash: TTD."])
	s.TimeToDie = function(unit)
		return 10
	end
end

-- 2T13 doubles the focus gain of Cobra Shot and Steady Shot.
-- following are some rudimentary functions to make the module
-- aware of that. Will probably get sacked when Mists goes live.
a.Wearing2T13 = false
local HunterT13 = {
	HeadSlot = {78793, 77030, 78698}, --[[Head]]
	ShoulderSlot = {78832, 77032, 78737},  --[[Shoulder]]
	ChestSlot = {78756, 77028, 78661},  --[[Chest]]
	LegsSlot = {78804, 77031, 78709}, --[[Legs]]
	HandsSlot = {78769, 77029, 78674}, --[[Gloves]]
}
a.scanFor2T13 = function()
	local count = 0
	for slot, list in pairs(HunterT13) do
		if s.Equipped(list, slot) then
			count = count + 1
		end
	end
	if count >= 2 then
		if not a.Wearing2T13 then
		end
		a.Wearing2T13 = true
	else
		if a.Wearing2T13 then
		end
		a.Wearing2T13 = false
	end
end
s.RegisterModuleEvent(AddonName, "PLAYER_EQUIPMENT_CHANGED", a.scanFor2T13)

a.MySelf = UnitGUID("player")

local LastMSTimestamp = 0
local CurrentTimestamp = 0
local LastMSTargetsHit = 0
local event, sourceGUID, spellId
a.COMBAT_LOG_EVENT_UNFILTERED = function (event, ...)
	CurrentTimestamp, event, _, sourceGUID, _, _, _, _, _, _, _, spellId = ...
	if sourceGUID == a.MySelf then
		if (event == "SPELL_DAMAGE" or event == "SPELL_MISSED") and spellId == 2643--[[Multi-Shot]] then
			if CurrentTimestamp < LastMSTimestamp + 1 then
				LastMSTargetsHit = LastMSTargetsHit + 1
			else
				LastMSTargetsHit = 1
				LastMSTimestamp = CurrentTimestamp
			end
		elseif a.PSFWatcherStarted and (event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESH") and spellId == 53220--[[Steady Focus]] then
			a.BuffPreSteadyFocus = false
		end
	end
end
s.RegisterModuleEvent(AddonName, "COMBAT_LOG_EVENT_UNFILTERED", a.COMBAT_LOG_EVENT_UNFILTERED)

-- Check if we are in AoE mode
a.AoE = function()
	if CurrentTimestamp < LastMSTimestamp + 3 then
		local spec = s.Spec()
		if spec == 3 then
			return (LastMSTargetsHit > 2)
		else
			return (LastMSTargetsHit > 5)
		end
	end
end
		
-- Handle Pre-Steady-Shot detection for MM hunters
a.BuffPreSteadyFocus = false
a.PSFWatcherStarted = false

--[[
Tested the following attacks and can confirm that they do not effect Steady Focus
- Dire Beast
- Fervor
- Lynx Rush
- Traps
]]
local ResetSteadyFocus = {
	53209--[[Chimera Shot]], 
	19434--[[Aimed Shot]], 
	3044--[[Arcane Shot]],
	5116--[[Concussive Shot]],
	20736--[[Distracting Shot]],
	53351--[[Kill Shot]],
	2643--[[Multi-Shot]],
	1978--[[Serpent Sting]],
	34490--[[Silencing Shot]],
	19801--[[Tranquilizing Shot]],
	82654--[[Widow Venom]],
	19503--[[Scatter Shot]],
	19386--[[Wyvern Sting]],
	109248--[[Binding Shot]],
	130392--[[Blink Strike]],
	131894--[[A Murder of Crows]],
	117050--[[Glaive Toss]],
	109259--[[Powershot]],
	120360--[[Barrage]],
}

a.PreSteadyFocus = function()
	if a.BuffPreSteadyFocus then
		return not s.Casting(56641--[[Steady Shot]], "player")
	else
		return s.Casting(56641--[[Steady Shot]], "player")
	end
end

a.stopPreSteadyFocus = function()
	if a.PSFWatcherStarted then
		s.UnregisterModuleEvent(AddonName, "UNIT_SPELLCAST_SUCCEEDED")
		a.PSFWatcherStarted = false
		a.BuffPreSteadyFocus = false
	end
end

a.startPreSteadyFocus = function()
	if not a.PSFWatcherStarted then
		s.RegisterModuleEvent(AddonName, "UNIT_SPELLCAST_SUCCEEDED", function (event, ...)
			local unit, _, _, _, spellID = ...
			if unit == "player" then
				if spellID == 56641--[[Steady Shot]] and not a.BuffPreSteadyFocus then
					a.BuffPreSteadyFocus = true
				elseif a.BuffPreSteadyFocus and tContains(ResetSteadyFocus, spellID)  then
					a.BuffPreSteadyFocus = false
				end
			end
		end)
		a.PSFWatcherStarted = true
	end
end

s.RegisterModuleEvent(AddonName, "ACTIVE_TALENT_GROUP_CHANGED", function (event, ...)
	if s.Spec() == 2 and not a.PSFWatcherStarted then
		a.startPreSteadyFocus()
	else
		a.stopPreSteadyFocus()
	end
end)

if s.Spec() == 2 and not a.PSFWatcherStarted then
	a.startPreSteadyFocus()
elseif s.Spec() == 2 and a.PSFWatcherStarted then
	a.BuffPreSteadyFocus = false
end
a.scanFor2T13()

s.RegisterModuleEvent(AddonName, "PLAYER_ENTERING_WORLD", function (event, ...)
	if s.Spec() == 2 and not a.PSFWatcherStarted then
		a.startPreSteadyFocus()
	elseif s.Spec() == 2 and a.PSFWatcherStarted then
		a.BuffPreSteadyFocus = false
	end
	a.scanFor2T13()
end)

-- Aimed Shot is better than Arcane Shot with extreme values of haste
-- following are some rudimentary functions to make the module
-- aware of that. May be better to scan for an actual haste breakpoint.
local HasteCDs = {
	49016--[[Unholy Frenzy]],
	80353--[[Time Warp]],
	2825--[[Bloodlust]],
	32182--[[Heroism]],
	90355--[[Ancient Hysteria]],
	26297--[[Berserking]],
	3045--[[Rapid Fire]],
	105919--[[Chronohunter]],
}
a.Hasted = function()
	return s.Buff(HasteCDs, "player")
end

-- Detect raid-wide haste buffs, so we know when to
-- use potions
local BloodlustSpells = {
	80353--[[Time Warp]],
	2825--[[Bloodlust]],
	32182--[[Heroism]],
	90355--[[Ancient Hysteria]],
}
a.Bloodlust = function()
	return s.Buff(BloodlustSpells, "player")
end

-- Determines if you will have enough Focus to cast
-- a Spell or List of Spells when they will come off CD
-- if you cast an Arcane Shot right now.
local tempCD, cdMin, cdMax
local regenRate
local ArcaneCost, tempCost, SpellCost
a.CanStillCast = function(SpellName)
	regenShot = 0
	-- How much will the Arcane Shot cost us?
	-- With Thrill of the Hunt, it may be for free
	ArcaneCost = s.SpellCost(3044--[[Arcane Shot]], SPELL_POWER_FOCUS)
	if s.Buff(34720--[[Thrill of the Hunt]], "player") then
		ArcaneCost = ArcaneCost - 20
	end
	
	-- If we are casting a Steady or Cobra right now, we will regain some
	-- Focus before we cast our Arcane Shot
	if s.Casting(77767--[[Cobra Shot]], "player") then
		regenShot = 14
	end
	if s.Casting(56641--[[Steady Shot]], "player") then
		regenShot = 14
	end
	if a.Wearing2T13 then
		regenShot = regenShot * 2
	end
	if s.Buff(53220--[[Steady Focus]], "player") and regenShot > 0 then
		regenShot = regenShot + 3
	end

	-- How much Focus do we regain naturally?
	_, regenRate = GetPowerRegen()
	regenRate = format("%.2f", regenRate)

	-- If we got a list of Spells, we have to check for the
	-- cost of each spell individually and for the sum as well
	if type(SpellName) == "table" then
		cdMin = 10
		cdMax = 0
		SpellCost = 0
		for _, SpellName in ipairs(SpellName) do
			tempCost = s.SpellCost(SpellName, SPELL_POWER_FOCUS)
			SpellCost = SpellCost + tempCost
			tempCD = s.SpellCooldown(SpellName)
			cdMin = min(cdMin, tempCD)
			cdMax = max(cdMax, tempCD)
			if (s.Power("player", SPELL_POWER_FOCUS) + tempCD * regenRate) < (tempCost + ArcaneCost - regenShot) then
				return false
			end
		end
	else
		SpellCost = s.SpellCost(SpellName, SPELL_POWER_FOCUS)
		cdMin = s.SpellCooldown(SpellName)
		cdMax = cdMin
	end
	if cdMin <= 1 then
		return false
	end
	if (s.Power("player", SPELL_POWER_FOCUS) + cdMax * regenRate) < (SpellCost + ArcaneCost - regenShot) then
		return false
	end
	return true
end

a.PetAssistActive = function()
    local name, active, exists
    local index = 10
    repeat
        name,_,_,_,active,_,_,exists = GetPetActionInfo(index)
        index = index - 1
        if name == "PET_MODE_ASSIST" then
			return active
        end
    until not exists
    return false
end